==> random-base-chi-0.0.5.tgz: package/README.md <==
<!--

@license Apache-2.0

Copyright (c) 2018 The Stdlib Authors.

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

   http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.

-->

# Chi Random Numbers

[![NPM version][npm-image]][npm-url] [![Build Status][test-image]][test-url] [![Coverage Status][coverage-image]][coverage-url] [![dependencies][dependencies-image]][dependencies-url]

> [Chi][chi] distributed pseudorandom numbers.

<section class="installation">

## Installation

```bash
npm install @stdlib/random-base-chi
```

</section>

<section class="usage">

## Usage

```javascript
var chi = require( '@stdlib/random-base-chi' );
```

#### chi( k )

Returns a pseudorandom number drawn from a [chi][chi] distribution with degrees of freedom `k`.

```javascript
var r = chi( 2 );
// returns <number>
```

If `k <= 0` or `k` is `NaN`, the function returns `NaN`.

```javascript
var r = chi( -2 );
// returns NaN

r = chi( NaN );
// returns NaN
```

#### chi.factory( \[k, ]\[options] )

Returns a pseudorandom number generator (PRNG) for generating pseudorandom numbers drawn from a [chi][chi] distribution.

```javascript
var rand = chi.factory();

var r = rand( 5 );
// returns <number>
```

If provided `k`, the returned generator returns random variates from the specified distribution.

```javascript
var rand = chi.factory( 1.0 );

var r = rand();
// returns <number>

r = rand();
// returns <number>
```

If not provided `k`, the returned generator requires that `k` be provided at each invocation.

```javascript
var rand = chi.factory();

var r = rand( 4 );
// returns <number>

r = rand( 3.14 );
// returns <number>
```

The function accepts the following `options`:

-   **prng**: pseudorandom number generator for generating uniformly distributed pseudorandom numbers on the interval `[0,1)`. If provided, the function **ignores** both the `state` and `seed` options. In order to seed the returned pseudorandom number generator, one must seed the provided `prng` (assuming the provided `prng` is seedable).
-   **seed**: pseudorandom number generator seed.
-   **state**: a [`Uint32Array`][@stdlib/array/uint32] containing pseudorandom number generator state. If provided, the function ignores the `seed` option.
-   **copy**: `boolean` indicating whether to copy a provided pseudorandom number generator state. Setting this option to `false` allows sharing state between two or more pseudorandom number generators. Setting this option to `true` ensures that a returned generator has exclusive control over its internal state. Default: `true`.

To use a custom PRNG as the underlying source of uniformly distributed pseudorandom numbers, set the `prng` option.

```javascript
var minstd = require( '@stdlib/random-base-minstd' );

var rand = chi.factory({
    'prng': minstd.normalized
});

var r = rand( 3 );
// returns <number>
```

To seed a pseudorandom number generator, set the `seed` option.

```javascript
var rand1 = chi.factory({
    'seed': 12345
});

var r1 = rand1( 3 );
// returns <number>

var rand2 = chi.factory( 3, {
    'seed': 12345
});

var r2 = rand2();
// returns <number>

var bool = ( r1 === r2 );
// returns true
```

To return a generator having a specific initial state, set the generator `state` option.

```javascript
var rand;
var bool;
var r;
var i;

// Generate pseudorandom numbers, thus progressing the generator state:
for ( i = 0; i < 1000; i++ ) {
    r = chi( 3 );
}

// Create a new PRNG initialized to the current state of `chi`:
rand = chi.factory({
    'state': chi.state
});

// Test that the generated pseudorandom numbers are the same:
bool = ( rand( 3 ) === chi( 3 ) );
// returns true
```

#### chi.NAME

The generator name.

```javascript
var str = chi.NAME;
// returns 'chi'
```

#### chi.PRNG

The underlying pseudorandom number generator.

```javascript
var prng = chi.PRNG;
// returns <Function>
```

#### chi.seed

The value used to seed `chi()`.

```javascript
var rand;
var r;
var i;

// Generate pseudorandom values...
for ( i = 0; i < 100; i++ ) {
    r = chi( 2.0 );
}

// Generate the same pseudorandom values...
rand = chi.factory( 2.0, {
    'seed': chi.seed
});
for ( i = 0; i < 100; i++ ) {
    r = rand();
}
```

If provided a PRNG for uniformly distributed numbers, this value is `null`.

<!-- eslint-disable stdlib/no-builtin-math -->

```javascript
var rand = chi.factory({
    'prng': Math.random
});

var seed = rand.seed;
// returns null
```

#### chi.seedLength

Length of generator seed.

```javascript
var len = chi.seedLength;
// returns <number>
```

If provided a PRNG for uniformly distributed numbers, this value is `null`.

<!-- eslint-disable stdlib/no-builtin-math -->

```javascript
var rand = chi.factory({
    'prng': Math.random
});

var len = rand.seedLength;
// returns null
```

#### chi.state

Writable property for getting and setting the generator state.

```javascript
var r = chi( 2 );
// returns <number>

r = chi( 2 );
// returns <number>

// ...

// Get a copy of the current state:
var state = chi.state;
// returns <Uint32Array>

r = chi( 2 );
// returns <number>

r = chi( 2 );
// returns <number>

// Reset the state:
chi.state = state;

// Replay the last two pseudorandom numbers:
r = chi( 2 );
// returns <number>

r = chi( 2 );
// returns <number>

// ...
```

If provided a PRNG for uniformly distributed numbers, this value is `null`.

<!-- eslint-disable stdlib/no-builtin-math -->

```javascript
var rand = chi.factory({
    'prng': Math.random
});

var state = rand.state;
// returns null
```

#### chi.stateLength

Length of generator state.

```javascript
var len = chi.stateLength;
// returns <number>
```

If provided a PRNG for uniformly distributed numbers, this value is `null`.

<!-- eslint-disable stdlib/no-builtin-math -->

```javascript
var rand = chi.factory({
    'prng': Math.random
});

var len = rand.stateLength;
// returns null
```

#### chi.byteLength

Size (in bytes) of generator state.

```javascript
var sz = chi.byteLength;
// returns <number>
```

If provided a PRNG for uniformly distributed numbers, this value is `null`.

<!-- eslint-disable stdlib/no-builtin-math -->

```javascript
var rand = chi.factory({
    'prng': Math.random
});

var sz = rand.byteLength;
// returns null
```

#### chi.toJSON()

Serializes the pseudorandom number generator as a JSON object.

```javascript
var o = chi.toJSON();
// returns { 'type': 'PRNG', 'name': '...', 'state': {...}, 'params': [] }
```

If provided a PRNG for uniformly distributed numbers, this method returns `null`.

<!-- eslint-disable stdlib/no-builtin-math -->

```javascript
var rand = chi.factory({
    'prng': Math.random
});

var o = rand.toJSON();
// returns null
```

</section>

<!-- /.usage -->

<section class="notes">

## Notes

-   If PRNG state is "shared" (meaning a state array was provided during PRNG creation and **not** copied) and one sets the generator state to a state array having a different length, the PRNG does **not** update the existing shared state and, instead, points to the newly provided state array. In order to synchronize PRNG output according to the new shared state array, the state array for **each** relevant PRNG must be **explicitly** set.
-   If PRNG state is "shared" and one sets the generator state to a state array of the same length, the PRNG state is updated (along with the state of all other PRNGs sharing the PRNG's state array).

</section>

<!-- /.notes -->

<section class="examples">

## Examples

<!-- eslint no-undef: "error" -->

```javascript
var chi = require( '@stdlib/random-base-chi' );

var seed;
var rand;
var i;

// Generate pseudorandom numbers...
for ( i = 0; i < 100; i++ ) {
    console.log( chi( 1.0 ) );
}

// Create a new pseudorandom number generator...
seed = 1234;
rand = chi.factory( 5.0, {
    'seed': seed
});
for ( i = 0; i < 100; i++ ) {
    console.log( rand() );
}

// Create another pseudorandom number generator using a previous seed...
rand = chi.factory( 1.0, {
    'seed': chi.seed
});
for ( i = 0; i < 100; i++ ) {
    console.log( rand() );
}
```

</section>

<!-- /.examples -->


<section class="main-repo" >

* * *

## Notice

This package is part of [stdlib][stdlib], a standard library for JavaScript and Node.js, with an emphasis on numerical and scientific computing. The library provides a collection of robust, high performance libraries for mathematics, statistics, streams, utilities, and more.

For more information on the project, filing bug reports and feature requests, and guidance on how to develop [stdlib][stdlib], see the main project [repository][stdlib].

#### Community

[![Chat][chat-image]][chat-url]

---

## License

See [LICENSE][stdlib-license].


## Copyright

Copyright &copy; 2016-2021. The Stdlib [Authors][stdlib-authors].

</section>

<!-- /.stdlib -->

<!-- Section for all links. Make sure to keep an empty line after the `section` element and another before the `/section` close. -->

<section class="links">

[npm-image]: http://img.shields.io/npm/v/@stdlib/random-base-chi.svg
[npm-url]: https://npmjs.org/package/@stdlib/random-base-chi

[test-image]: https://github.com/stdlib-js/random-base-chi/actions/workflows/test.yml/badge.svg
[test-url]: https://github.com/stdlib-js/random-base-chi/actions/workflows/test.yml

[coverage-image]: https://img.shields.io/codecov/c/github/stdlib-js/random-base-chi/main.svg
[coverage-url]: https://codecov.io/github/stdlib-js/random-base-chi?branch=main

[dependencies-image]: https://img.shields.io/david/stdlib-js/random-base-chi.svg
[dependencies-url]: https://david-dm.org/stdlib-js/random-base-chi/main

[chat-image]: https://img.shields.io/gitter/room/stdlib-js/stdlib.svg
[chat-url]: https://gitter.im/stdlib-js/stdlib/

[stdlib]: https://github.com/stdlib-js/stdlib

[stdlib-authors]: https://github.com/stdlib-js/stdlib/graphs/contributors

[stdlib-license]: https://raw.githubusercontent.com/stdlib-js/random-base-chi/main/LICENSE

[chi]: https://en.wikipedia.org/wiki/Chi_distribution

[@stdlib/array/uint32]: https://www.npmjs.com/package/@stdlib/array-uint32

</section>

<!-- /.links -->

==> random-base-chi-0.0.6.tgz: package/README.md <==
<!--

@license Apache-2.0

Copyright (c) 2018 The Stdlib Authors.

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

   http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.

-->

# Chi Random Numbers

[![NPM version][npm-image]][npm-url] [![Build Status][test-image]][test-url] [![Coverage Status][coverage-image]][coverage-url] <!-- [![dependencies][dependencies-image]][dependencies-url] -->

> [Chi][chi] distributed pseudorandom numbers.

<section class="installation">

## Installation

```bash
npm install @stdlib/random-base-chi
```

</section>

<section class="usage">

## Usage

```javascript
var chi = require( '@stdlib/random-base-chi' );
```

#### chi( k )

Returns a pseudorandom number drawn from a [chi][chi] distribution with degrees of freedom `k`.

```javascript
var r = chi( 2 );
// returns <number>
```

If `k <= 0` or `k` is `NaN`, the function returns `NaN`.

```javascript
var r = chi( -2 );
// returns NaN

r = chi( NaN );
// returns NaN
```

#### chi.factory( \[k, ]\[options] )

Returns a pseudorandom number generator (PRNG) for generating pseudorandom numbers drawn from a [chi][chi] distribution.

```javascript
var rand = chi.factory();

var r = rand( 5 );
// returns <number>
```

If provided `k`, the returned generator returns random variates from the specified distribution.

```javascript
var rand = chi.factory( 1.0 );

var r = rand();
// returns <number>

r = rand();
// returns <number>
```

If not provided `k`, the returned generator requires that `k` be provided at each invocation.

```javascript
var rand = chi.factory();

var r = rand( 4 );
// returns <number>

r = rand( 3.14 );
// returns <number>
```

The function accepts the following `options`:

-   **prng**: pseudorandom number generator for generating uniformly distributed pseudorandom numbers on the interval `[0,1)`. If provided, the function **ignores** both the `state` and `seed` options. In order to seed the returned pseudorandom number generator, one must seed the provided `prng` (assuming the provided `prng` is seedable).
-   **seed**: pseudorandom number generator seed.
-   **state**: a [`Uint32Array`][@stdlib/array/uint32] containing pseudorandom number generator state. If provided, the function ignores the `seed` option.
-   **copy**: `boolean` indicating whether to copy a provided pseudorandom number generator state. Setting this option to `false` allows sharing state between two or more pseudorandom number generators. Setting this option to `true` ensures that a returned generator has exclusive control over its internal state. Default: `true`.

To use a custom PRNG as the underlying source of uniformly distributed pseudorandom numbers, set the `prng` option.

```javascript
var minstd = require( '@stdlib/random-base-minstd' );

var rand = chi.factory({
    'prng': minstd.normalized
});

var r = rand( 3 );
// returns <number>
```

To seed a pseudorandom number generator, set the `seed` option.

```javascript
var rand1 = chi.factory({
    'seed': 12345
});

var r1 = rand1( 3 );
// returns <number>

var rand2 = chi.factory( 3, {
    'seed': 12345
});

var r2 = rand2();
// returns <number>

var bool = ( r1 === r2 );
// returns true
```

To return a generator having a specific initial state, set the generator `state` option.

```javascript
var rand;
var bool;
var r;
var i;

// Generate pseudorandom numbers, thus progressing the generator state:
for ( i = 0; i < 1000; i++ ) {
    r = chi( 3 );
}

// Create a new PRNG initialized to the current state of `chi`:
rand = chi.factory({
    'state': chi.state
});

// Test that the generated pseudorandom numbers are the same:
bool = ( rand( 3 ) === chi( 3 ) );
// returns true
```

#### chi.NAME

The generator name.

```javascript
var str = chi.NAME;
// returns 'chi'
```

#### chi.PRNG

The underlying pseudorandom number generator.

```javascript
var prng = chi.PRNG;
// returns <Function>
```

#### chi.seed

The value used to seed `chi()`.

```javascript
var rand;
var r;
var i;

// Generate pseudorandom values...
for ( i = 0; i < 100; i++ ) {
    r = chi( 2.0 );
}

// Generate the same pseudorandom values...
rand = chi.factory( 2.0, {
    'seed': chi.seed
});
for ( i = 0; i < 100; i++ ) {
    r = rand();
}
```

If provided a PRNG for uniformly distributed numbers, this value is `null`.

<!-- eslint-disable stdlib/no-builtin-math -->

```javascript
var rand = chi.factory({
    'prng': Math.random
});

var seed = rand.seed;
// returns null
```

#### chi.seedLength

Length of generator seed.

```javascript
var len = chi.seedLength;
// returns <number>
```

If provided a PRNG for uniformly distributed numbers, this value is `null`.

<!-- eslint-disable stdlib/no-builtin-math -->

```javascript
var rand = chi.factory({
    'prng': Math.random
});

var len = rand.seedLength;
// returns null
```

#### chi.state

Writable property for getting and setting the generator state.

```javascript
var r = chi( 2 );
// returns <number>

r = chi( 2 );
// returns <number>

// ...

// Get a copy of the current state:
var state = chi.state;
// returns <Uint32Array>

r = chi( 2 );
// returns <number>

r = chi( 2 );
// returns <number>

// Reset the state:
chi.state = state;

// Replay the last two pseudorandom numbers:
r = chi( 2 );
// returns <number>

r = chi( 2 );
// returns <number>

// ...
```

If provided a PRNG for uniformly distributed numbers, this value is `null`.

<!-- eslint-disable stdlib/no-builtin-math -->

```javascript
var rand = chi.factory({
    'prng': Math.random
});

var state = rand.state;
// returns null
```

#### chi.stateLength

Length of generator state.

```javascript
var len = chi.stateLength;
// returns <number>
```

If provided a PRNG for uniformly distributed numbers, this value is `null`.

<!-- eslint-disable stdlib/no-builtin-math -->

```javascript
var rand = chi.factory({
    'prng': Math.random
});

var len = rand.stateLength;
// returns null
```

#### chi.byteLength

Size (in bytes) of generator state.

```javascript
var sz = chi.byteLength;
// returns <number>
```

If provided a PRNG for uniformly distributed numbers, this value is `null`.

<!-- eslint-disable stdlib/no-builtin-math -->

```javascript
var rand = chi.factory({
    'prng': Math.random
});

var sz = rand.byteLength;
// returns null
```

#### chi.toJSON()

Serializes the pseudorandom number generator as a JSON object.

```javascript
var o = chi.toJSON();
// returns { 'type': 'PRNG', 'name': '...', 'state': {...}, 'params': [] }
```

If provided a PRNG for uniformly distributed numbers, this method returns `null`.

<!-- eslint-disable stdlib/no-builtin-math -->

```javascript
var rand = chi.factory({
    'prng': Math.random
});

var o = rand.toJSON();
// returns null
```

</section>

<!-- /.usage -->

<section class="notes">

## Notes

-   If PRNG state is "shared" (meaning a state array was provided during PRNG creation and **not** copied) and one sets the generator state to a state array having a different length, the PRNG does **not** update the existing shared state and, instead, points to the newly provided state array. In order to synchronize PRNG output according to the new shared state array, the state array for **each** relevant PRNG must be **explicitly** set.
-   If PRNG state is "shared" and one sets the generator state to a state array of the same length, the PRNG state is updated (along with the state of all other PRNGs sharing the PRNG's state array).

</section>

<!-- /.notes -->

<section class="examples">

## Examples

<!-- eslint no-undef: "error" -->

```javascript
var chi = require( '@stdlib/random-base-chi' );

var seed;
var rand;
var i;

// Generate pseudorandom numbers...
for ( i = 0; i < 100; i++ ) {
    console.log( chi( 1.0 ) );
}

// Create a new pseudorandom number generator...
seed = 1234;
rand = chi.factory( 5.0, {
    'seed': seed
});
for ( i = 0; i < 100; i++ ) {
    console.log( rand() );
}

// Create another pseudorandom number generator using a previous seed...
rand = chi.factory( 1.0, {
    'seed': chi.seed
});
for ( i = 0; i < 100; i++ ) {
    console.log( rand() );
}
```

</section>

<!-- /.examples -->

<!-- Section for related `stdlib` packages. Do not manually edit this section, as it is automatically populated. -->

<section class="related">

</section>

<!-- /.related -->

<!-- Section for all links. Make sure to keep an empty line after the `section` element and another before the `/section` close. -->


<section class="main-repo" >

* * *

## Notice

This package is part of [stdlib][stdlib], a standard library for JavaScript and Node.js, with an emphasis on numerical and scientific computing. The library provides a collection of robust, high performance libraries for mathematics, statistics, streams, utilities, and more.

For more information on the project, filing bug reports and feature requests, and guidance on how to develop [stdlib][stdlib], see the main project [repository][stdlib].

#### Community

[![Chat][chat-image]][chat-url]

---

## License

See [LICENSE][stdlib-license].


## Copyright

Copyright &copy; 2016-2022. The Stdlib [Authors][stdlib-authors].

</section>

<!-- /.stdlib -->

<!-- Section for all links. Make sure to keep an empty line after the `section` element and another before the `/section` close. -->

<section class="links">

[npm-image]: http://img.shields.io/npm/v/@stdlib/random-base-chi.svg
[npm-url]: https://npmjs.org/package/@stdlib/random-base-chi

[test-image]: https://github.com/stdlib-js/random-base-chi/actions/workflows/test.yml/badge.svg
[test-url]: https://github.com/stdlib-js/random-base-chi/actions/workflows/test.yml

[coverage-image]: https://img.shields.io/codecov/c/github/stdlib-js/random-base-chi/main.svg
[coverage-url]: https://codecov.io/github/stdlib-js/random-base-chi?branch=main

<!--

[dependencies-image]: https://img.shields.io/david/stdlib-js/random-base-chi.svg
[dependencies-url]: https://david-dm.org/stdlib-js/random-base-chi/main

-->

[umd]: https://github.com/umdjs/umd
[es-module]: https://developer.mozilla.org/en-US/docs/Web/JavaScript/Guide/Modules

[deno-url]: https://github.com/stdlib-js/random-base-chi/tree/deno
[umd-url]: https://github.com/stdlib-js/random-base-chi/tree/umd
[esm-url]: https://github.com/stdlib-js/random-base-chi/tree/esm

[chat-image]: https://img.shields.io/gitter/room/stdlib-js/stdlib.svg
[chat-url]: https://gitter.im/stdlib-js/stdlib/

[stdlib]: https://github.com/stdlib-js/stdlib

[stdlib-authors]: https://github.com/stdlib-js/stdlib/graphs/contributors

[stdlib-license]: https://raw.githubusercontent.com/stdlib-js/random-base-chi/main/LICENSE

[chi]: https://en.wikipedia.org/wiki/Chi_distribution

[@stdlib/array/uint32]: https://www.npmjs.com/package/@stdlib/array-uint32

</section>

<!-- /.links -->
